import uuid, hashlib
from pathlib import Path
from psycopg2.extras import execute_batch
from .imports import *
from ..db import (
    insert_document,

    get_conn
    
)

ROOT = Path("/srv/staging")


# -------------------------
# PATH HELPERS
# -------------------------
def rel(p: Path):
    return str(p.relative_to(ROOT))

def page_key(p: Path):
    name = p.name

    if not name.startswith("page_"):
        return None

    num = name.replace("page_", "")

    if not num.isdigit():
        return None

    return int(num)

def generate_doc_id(base_path: str):
    return hashlib.sha1(base_path.encode()).hexdigest()

def find_pdf(base_dir: Path):
    for p in Path(base_dir).glob("*.pdf"):
        return p
    return None
# -------------------------
# PAGE EXTRACTION
# -------------------------
def extract_pages(base_dir):
    pages = []
    pages_dir = Path(base_dir) / "pages"

    valid_pages = []

    for p in pages_dir.glob("page_*"):
        if not p.is_dir():
            continue

        key = page_key(p)
        if key is None:
            print(f"⚠️ Skipping invalid page dir: {p}")
            continue

        valid_pages.append((p, key))

    # sort safely
    valid_pages.sort(key=lambda x: x[1])

    for page_dir, page_number in valid_pages:
        image = page_dir / "image.png"
        html = page_dir / "index.html"
        text = page_dir / "text.txt"
        info = page_dir / "info.json"

        pages.append({
            "page_number": page_number,
            "page_path": rel(page_dir),

            "image_path": rel(image) if image.exists() else None,
            "html_path": rel(html) if html.exists() else None,
            "text_path": rel(text) if text.exists() else None,
            "info_path": rel(info) if info.exists() else None,

            "has_image": image.exists(),
            "has_html": html.exists(),
            "has_text": text.exists(),
            "has_info": info.exists(),
        })

    return pages

### -------------------------
### PAGE INSERT
### -------------------------
##def insert_pages(cur, doc_id, pages):
##    query = """
##        INSERT INTO document_pages (
##            document_id,
##            page_number,
##            page_path,
##            image_path,
##            html_path,
##            text_path,
##            info_path,
##            has_image,
##            has_html,
##            has_text,
##            has_info
##        )
##        VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)
##    """
##
##    data = [
##        (
##            doc_id,
##            p["page_number"],
##            p["page_path"],
##            p["image_path"],
##            p["html_path"],
##            p["text_path"],
##            p["info_path"],
##            p["has_image"],
##            p["has_html"],
##            p["has_text"],
##            p["has_info"],
##        )
##        for p in pages
##    ]
##
##    for row in data:
##        cur.execute(query, row)
##

# -------------------------
# BUILD DOCUMENT PAYLOAD
# -------------------------
def build_document_payload(base_dir):
    base_dir = Path(base_dir)

    pdf = find_pdf(base_dir)

    return {
        "doc_id": generate_doc_id(rel(base_dir)),

        # simple deterministic slug
        "slug": base_dir.name.lower().replace(" ", "-"),

        "base_path": rel(base_dir),
        "pdf_path": rel(pdf) if pdf else None,

        # 🔥 NO ANALYSIS HERE
        "pages": extract_pages(base_dir),
    }

# -------------------------
# MAIN INGEST FUNCTION
# -------------------------
def upload_document(base_dir):
    conn = get_conn()
    cur = conn.cursor()

    try:
        payload = build_document_payload(base_dir)

        doc_id = insert_document(payload)

        if not doc_id:
            print("⏭️ Skipping (already exists)")
            return None

        # 🔥 THIS is key
        insert_pages(cur, doc_id, payload["pages"])

        conn.commit()

        print(f"✅ Uploaded document {doc_id}")
        return doc_id

    except Exception as e:
        conn.rollback()
        raise e

    finally:
        cur.close()
        conn.close()



from .imports import *

def discover_images(config: SiteConfig):
    """
    Yield one record per image leaf directory.
    Expected layout:
        {slug}/{slug}.webp (or .jpg/.png)
        {slug}/info.json
        {slug}/index.html
    """
    dirs,files = get_files_and_dirs(config.imgs_dir,allowed_exts=['.json'])
    info_paths = [file for file in files if file.endswith('info.json')]
    
    for info_path in info_paths:
        correct_urls(info_path)
        html_path = info_path.replace('info.json', "index.html")
        img_dir = os.path.dirname(info_path)
        slug = os.path.basename(img_dir)
        img_path = get_best_image(img_dir)
        if img_path:
            img_base = os.path.splitext(os.path.basename(img_path))[0]
            if img_base != slug:
                continue

            

            info = safe_load_json(info_path)
            title = info.get("title") or humanize(slug)
            description = clean_text(
                info.get("longdesc") or info.get("alt") or info.get("caption") or ""
            )
            keywords = normalize_keywords(info.get("keywords_str", ""))
            schema = info.get("schema") or {}
            thumb_path = first_existing_file(
                os.path.join(img_dir, f"{slug}_thumb.webp"),
                os.path.join(img_dir, f"{slug}_thumb.jpg"),
                os.path.join(img_dir, f"{slug}_resized.png"),
                img_path,  # the image is its own thumbnail by default
            )

            yield make_record(
                kind="image",
                slug=slug,
                dir_path=img_dir,
                title=title,
                description=description,
                keywords=keywords,
                source_path=img_path,
                html_path=html_path,
                info_path=info_path,
                thumbnail_path=thumb_path,   # <-- new
                info=info,
                schema=schema,
            )


def discover_videos(config):
    """
    Yield one record per video leaf directory.
    Expected layout:
        {slug}/{slug}.mp4
        {slug}/info.json
        {slug}/thumbnails/{slug}_frame_N.jpg
        {slug}/captions.srt
    """
    for vid_path in find_files_by_ext(config.videos_dir, set(config.video_exts)):
        vid_dir = os.path.dirname(vid_path)
        slug = os.path.basename(vid_dir)

        info_path = os.path.join(vid_dir, "info.json")
        html_path = os.path.join(vid_dir, "index.html")
        info = safe_load_json(info_path)

        title = clean_text(
            info.get("seo_title") or humanize(slug), 120
        )
        description = clean_text(
            info.get("seo_description") or info.get("summary") or "", 200
        )
        keywords = normalize_keywords(
            info.get("seo_tags")
            or info.get("keywords")
            or info.get("combined_keywords")
            or []
        )
        duration = info.get("duration_formatted", "")
        resolution = (info.get("file_metadata") or {}).get("resolution", "")

        # thumbnail
        thumb_dir = os.path.join(vid_dir, "thumbnails")
        thumb_path = ""
        if os.path.isdir(thumb_dir):
            pattern = "%s_frame_0.jpg" % slug
            candidate = os.path.join(thumb_dir, pattern)
            if os.path.isfile(candidate):
                thumb_path = candidate

        # fallback thumbnail from info.json
        if not thumb_path:
            raw_thumb = (info.get("thumbnail") or {}).get("file_path", "")
            if raw_thumb and os.path.isfile(raw_thumb):
                thumb_path = raw_thumb

        # poster
        poster_path = first_existing_file(
            os.path.join(vid_dir, "poster.jpg"),
            os.path.join(vid_dir, "poster.png"),
            os.path.join(vid_dir, "thumbnail.jpg"),
            os.path.join(vid_dir, "thumbnail.png"),
        )

        # captions
        captions_path = first_existing_file(
            os.path.join(vid_dir, "captions.srt"),
            os.path.join(vid_dir, "captions.vtt"),
        )

        yield make_record(
            kind="video",
            slug=slug,
            dir_path=vid_dir,
            title=title,
            description=description,
            keywords=keywords,
            source_path=vid_path,
            html_path=html_path,
            info_path=info_path,
            info=info,
            thumbnail_path=thumb_path,
            poster_path=poster_path,
            captions_path=captions_path,
            duration=duration,
            resolution=resolution,
        )


def discover_all(config):
    """Yield all media records across all types."""
    yield from discover_pdfs(config)
    yield from discover_images(config)
    yield from discover_videos(config)



from abstract_utilities import *
PDF_DIRECTORY = directory = PDFS_ROOT
input(PDFS_ROOT)
dirs,ALL_PDFS = get_files_and_dirs(PDF_DIRECTORY,allowed_exts=".pdf")


pdf_dirname = os.path.dirname(pdf_path)
pages_dir = os.path.join(pdf_dirname,'pages')

render_pdf_pages(
    pdf_path: Path(pdf_path),
    output_dir: Path(pages_dir),
    dpi= 200,
    fmt= "png",
)


for page in os.listdir(pages_dir):
    page_image_path = os.path.join(pages_dir,page,'image.png')
    page_text = image_to_text(
        image_path=page_image_path
        )
    page_text_path = os.path.join(pages_dir,page,'text.txt')
    write_to_file(file_path=page_text_path,contents=page_text)
    analyzed_data = analyze_page(text_path = page_text_path)
    page_info_path = os.path.join(pages_dir,page,'info.json')
    safe_dump_to_json(file_path=page_info_path,data=analyzed_data)
    analyzed_data['image_path']=page_image_path
    meta_info = get_meta_info(analyzed_data)




from .imports import *
def slice_pdf(
        pdf_path:         str,
        out_root:         Optional[str] = None,
        engines                         = "layout_ocr",
        engine_directory: bool          = False,
        visualize:        bool          = None,
        root_url                        = None,
        media_root                      = None,
        pdfs_public_url                 = None,
        image_strategies = None,
    ):
    out_root = os.path.dirname(pdf_path)
    data = {"pdf_path":pdf_path,
                    "out_root":out_root,
                    "engines":engines,
                    "engine_directory":engine_directory,
                    "visualize":visualize,
                    "root_url":ROOT_URL,
                    "media_root":MEDIA_ROOT,
                    "pdfs_public_url":PDFS_PUBLIC_URL,
                    "image_strategies":image_strategies
                }
    return postRequest('https://clownworld.biz/ocr/pdfs/manifest/process',data=data)



from .imports import *
from .assembly import *
from .directory_walker import *
from .helpers import *
from .rendering import *
from .template_registry import *
# ═══════════════════════════════════════════════════════════════
# Entry point — wires everything, then walks
# ═══════════════════════════════════════════════════════════════


@dataclass
class CLIConfig:
    dsn: str
    tenant_slug: str
    root: Path
    media_root: Path
    base_url: str
    site_root: str
    template_dir: Path
    no_recurse: bool
    dry_run: bool


def run(cfg: CLIConfig) -> None:
    conn = psycopg.connect(cfg.dsn, autocommit=True)
    try:
        repo = Repository(conn)
        seo_repo = SeoRepository(conn)

        tenant = repo.get_tenant_by_slug(cfg.tenant_slug)
        if tenant is None:
            print(f"ERROR: no tenant with slug={cfg.tenant_slug!r}", file=sys.stderr)
            sys.exit(1)

        templates = TemplateRegistry(cfg.template_dir)
        resolver = PathResolver(media_root=cfg.media_root, site_root=cfg.site_root)
        doc_index = DocumentIndex(repo, seo_repo, tenant.id)

        walk_config = WalkConfig(
            site_root=cfg.site_root,
            dry_run=cfg.dry_run,
            templates=templates,
            resolver=resolver,
            doc_index=doc_index,
        )

        generate_for_directory(
            directory=cfg.root,
            base_url=cfg.base_url,
            config=walk_config,
            recurse=not cfg.no_recurse,
        )
    finally:
        conn.close()


def generate_htmls_main() -> None:
    p = argparse.ArgumentParser(
        description="Generate static HTML for every directory from the document registry DB.",
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    p.add_argument(
        "--dsn",
        default=os.environ.get("DATABASE_URL", ""),
        help="PostgreSQL DSN (or set DATABASE_URL env var)",
    )
    p.add_argument("--tenant-slug", required=True, dest="tenant_slug")
    p.add_argument(
        "--root", required=True,
        help="Filesystem root to walk (e.g. /srv/media/thedailydialectics/pdfs)",
    )
    p.add_argument(
        "--media-root", required=True, dest="media_root",
        help="Top-level media root for path→URL conversion",
    )
    p.add_argument(
        "--base-url", required=True, dest="base_url",
        help="Public URL for --root (e.g. https://thedailydialectics.com/pdfs)",
    )
    p.add_argument(
        "--site-root",
        default=SITE_ROOT_DEFAULT, dest="site_root",
    )
    p.add_argument(
        "--template-dir",
        default=None, dest="template_dir",
        help="Path to Jinja2 templates (default: ./templates/ next to this script)",
    )
    p.add_argument("--no-recurse", action="store_true", dest="no_recurse")
    p.add_argument("--dry-run", action="store_true", dest="dry_run")
    args = p.parse_args()

    if not args.dsn:
        print("ERROR: --dsn or DATABASE_URL required", file=sys.stderr)
        sys.exit(1)

    template_dir = Path(args.template_dir) if args.template_dir else (
        Path(__file__).resolve().parent / "templates"
    )

    cfg = CLIConfig(
        dsn=args.dsn,
        tenant_slug=args.tenant_slug,
        root=Path(args.root).resolve(),
        media_root=Path(args.media_root).resolve(),
        base_url=args.base_url.rstrip("/"),
        site_root=args.site_root.rstrip("/"),
        template_dir=template_dir,
        no_recurse=args.no_recurse,
        dry_run=args.dry_run,
    )

    if not cfg.root.is_dir():
        print(f"ERROR: {cfg.root} is not a directory", file=sys.stderr)
        sys.exit(1)

    run(cfg)



from .imports import *
def analyze_page(pdf_dir,page_index):
    filtered_text = get_filtered_text(pdf_dir,page_index)
    text = truncate_to_word_limit(filtered_text, max_words=510)
    data={"text":text,
    "scope":f"page:{page_index}",
    "summary_preset":"brief",         # one page ≠ article-length
    "keyword_preset":"seo"}
    logger.info(f"analyze_page data = {data}")
    response = postRequest('https://clownworld.biz/hugpy/analyze/text',data=data)
    logger.info(f"response = {response}")
    return response
def analyze_full(pdf_dir):
    filtered_text = get_unfiltered_text(pdf_dir)
    text = truncate_to_word_limit(filtered_text, max_words=510)
    data={"text":text,
    "scope":f"full",
    "summary_preset":"article",         # one page ≠ article-length
    "keyword_preset":"seo"}
    return postRequest('https://clownworld.biz/hugpy/analyze/text',data=data)

def refine_keywords(text, preset="seo"):
    data={"text":text,"preset":preset}
    return postRequest('https://clownworld.biz/hugpy/keybert/refine_keywords',data=data)
def summarize(text, preset="article"):
    data={"text":text,"preset":preset}
    return postRequest('https://clownworld.biz/hugpy/summarizer/summarize',data=data)



from imports.src.abstract_react.meta_utils.apis.metadata.metadata import *
image = "/home/op/Documents/python/modules/src/modules/abstract_react/tests/test_image/abstractendeavors.webp"
info = {"thumbnail":image}
base_image_dir = "/home/op/Documents/python/modules/src/modules/abstract_react/tests/test_image/"
base_image_url = "https://thedailydialectics.com/image"
meta_data = get_meta_info(info,base_image_dir=base_image_dir,base_image_url=base_image_url)
input(meta_data)



from .process_registry import *
from .imports import dataclass,os
# ---------------------------------------------------------------------------
# 7. SITEMAP GENERATION — from enriched records
# ---------------------------------------------------------------------------

@dataclass
class SitemapUrl:
    loc: str
    lastmod: str = ""
    changefreq: str = ""
    priority: str = ""


def infer_changefreq_and_priority(url):
    """Assign changefreq/priority based on URL path prefix."""
    path = url.split("://", 1)[-1]
    path = "/" + path.split("/", 1)[1] if "/" in path else "/"
    clean = path.strip("/")
    if clean == "":
        return "daily", "1.0"
    if clean.startswith("pdfs"):
        return "weekly", "0.9"
    if clean.startswith("imgs"):
        return "weekly", "0.8"
    if clean.startswith("videos"):
        return "weekly", "0.8"
    return "monthly", "0.7"


def records_to_sitemap_urls(records, config):
    """Convert enriched records into deduplicated SitemapUrl list."""
    seen = {}
    for rec in records:
        loc = rec.get("canonical_url") or rec.get("dir_url", "")
        if not loc:
            continue

        changefreq, priority = infer_changefreq_and_priority(loc)

        # best lastmod: html file mtime, then source file mtime
        lastmod = (
            iso_date_from_mtime(rec.get("html_path"))
            or iso_date_from_mtime(rec.get("metadata_path"))
            or iso_date_from_mtime(rec.get("source_path"))
            or ""
        )

        entry = SitemapUrl(loc=loc, lastmod=lastmod, changefreq=changefreq, priority=priority)

        existing = seen.get(loc)
        if existing is None or (entry.lastmod or "") > (existing.lastmod or ""):
            seen[loc] = entry

    return sorted(seen.values(), key=lambda x: x.loc)


def render_sitemap_xml(urls):
    """Render a list of SitemapUrl into sitemap XML."""
    lines = [
        '<?xml version="1.0" encoding="UTF-8"?>',
        '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">',
    ]
    for url in urls:
        if not url.loc:
            continue
        lines.append("  <url>")
        lines.append("    <loc>%s</loc>" % xml_escape(url.loc))
        if url.lastmod:
            lines.append("    <lastmod>%s</lastmod>" % url.lastmod)
        if url.changefreq:
            lines.append("    <changefreq>%s</changefreq>" % url.changefreq)
        if url.priority:
            lines.append("    <priority>%s</priority>" % url.priority)
        lines.append("  </url>")
    lines.append("</urlset>")
    lines.append("")
    return "\n".join(lines)


def render_sitemap_index_xml(base_url, sitemap_names, output_dir):
    """Render a sitemap index XML."""
    today = datetime.now(timezone.utc).date().isoformat()
    lines = [
        '<?xml version="1.0" encoding="UTF-8"?>',
        '<sitemapindex xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">',
    ]
    for name in sitemap_names:
        path = os.path.join(output_dir, name)
        lastmod = iso_date_from_mtime(path) or today
        lines.append("  <sitemap>")
        lines.append("    <loc>%s</loc>" % xml_escape(base_url.rstrip("/") + "/" + name))
        lines.append("    <lastmod>%s</lastmod>" % lastmod)
        lines.append("  </sitemap>")
    lines.append("</sitemapindex>")
    lines.append("")
    return "\n".join(lines)


def write_sitemaps(records, config, output_dir):
    """
    Write sitemap.xml (or split sitemaps + index) from records.
    Returns list of written file paths.
    """
    urls = records_to_sitemap_urls(records, config)
    os.makedirs(output_dir, exist_ok=True)
    written = []

    if len(urls) <= config.max_sitemap_urls:
        out_path = os.path.join(output_dir, "sitemap.xml")
        safe_write_text(out_path, render_sitemap_xml(urls))
        written.append(out_path)
        return written

    # split into chunks
    chunk_size = config.max_sitemap_urls
    sitemap_names = []
    for idx in range(0, len(urls), chunk_size):
        batch = urls[idx:idx + chunk_size]
        name = "sitemap-%d.xml" % (idx // chunk_size + 1)
        out_path = os.path.join(output_dir, name)
        safe_write_text(out_path, render_sitemap_xml(batch))
        written.append(out_path)
        sitemap_names.append(name)

    index_path = os.path.join(output_dir, "sitemap.xml")
    safe_write_text(
        index_path,
        render_sitemap_index_xml(config.site_root, sitemap_names, output_dir),
    )
    written.append(index_path)
    return written



from dataclasses import dataclass, field
from typing import Optional
import json
import html as html_lib

from .imports import *
# ─── Config ───────────────────────────────────────────────────────────

@dataclass(frozen=True)
class SiteConfig:
    domain: str = DOMAIN
    url: str = ROOT_URL
    creator: str = SITE_NAME
    author: str = f"@{SITE_NAME}"
    twitter_site: str = f"@{SITE_NAME}"
    theme_color: str = "#FFFFFF"
    color_scheme: str = "light"
    charset: str = "UTF-8"
    license: str = "CC BY-SA 4.0"


DEFAULT_SITE = SiteConfig()


# ─── Schema ───────────────────────────────────────────────────────────

@dataclass
class PageMeta:
    title: str
    description: str
    keywords: list[str]
    canonical_url: str
    image_url: str
    page_url: str
    site: SiteConfig = field(default_factory=lambda: DEFAULT_SITE)
    og_type: str = "article"
    twitter_card: str = "summary_large_image"
    robots: str = "index, follow"
    image_alt: Optional[str] = None
    locale: str = "en_US"

    def __post_init__(self):
        # kill None-as-string at the source
        self.title = self.title or ""
        self.description = (self.description or "")[:300]
        self.image_alt = self.image_alt or self.title
        if isinstance(self.keywords, str):
            self.keywords = [k.strip() for k in self.keywords.split(",") if k.strip()]

    @property
    def keywords_str(self) -> str:
        return ", ".join(self.keywords)


# ─── Single meta builder ─────────────────────────────────────────────

def _esc(val: str) -> str:
    return html_lib.escape(str(val), quote=True)


def render_meta_tags(meta: PageMeta) -> str:
    s = meta.site
    tags = [
        f'<meta charset="{s.charset}">',
        '<meta name="viewport" content="width=device-width, initial-scale=1">',
        f'<meta name="theme-color" content="{s.theme_color}">',
        f'<meta name="color-scheme" content="{s.color_scheme}">',
        f"<title>{_esc(meta.title)} | {s.domain}</title>",
        f'<meta name="description" content="{_esc(meta.description)}">',
        f'<meta name="keywords" content="{_esc(meta.keywords_str)}">',
        f'<meta name="robots" content="{meta.robots}">',
        f'<meta name="author" content="{_esc(s.author)}">',
        # og
        f'<meta property="og:type" content="{meta.og_type}">',
        f'<meta property="og:title" content="{_esc(meta.title)}">',
        f'<meta property="og:description" content="{_esc(meta.description)}">',
        f'<meta property="og:url" content="{_esc(meta.page_url)}">',
        f'<meta property="og:image" content="{_esc(meta.image_url)}">',
        f'<meta property="og:image:alt" content="{_esc(meta.image_alt)}">',
        f'<meta property="og:site_name" content="{_esc(s.domain)}">',
        f'<meta property="og:locale" content="{meta.locale}">',
        # twitter
        f'<meta name="twitter:card" content="{meta.twitter_card}">',
        f'<meta name="twitter:title" content="{_esc(meta.title)}">',
        f'<meta name="twitter:description" content="{_esc(meta.description)}">',
        f'<meta name="twitter:image" content="{_esc(meta.image_url)}">',
        f'<meta name="twitter:site" content="{_esc(s.twitter_site)}">',
        f'<meta name="twitter:creator" content="{_esc(s.creator)}">',
        # canonical
        f'<link rel="canonical" href="{_esc(meta.canonical_url)}">',
    ]
    return "\n  ".join(tags)


def render_schema_ld(meta: PageMeta, schema_type: str = "CreativeWork", **extra) -> str:
    schema = {
        "@context": "https://schema.org",
        "@type": schema_type,
        "name": meta.title,
        "description": meta.description,
        "keywords": meta.keywords_str,
        "url": meta.canonical_url,
        "thumbnailUrl": meta.image_url,
        **extra,
    }
    return f'<script type="application/ld+json">{json.dumps(schema, ensure_ascii=False)}</script>'



# ─── Factory: build PageMeta from the data dicts you already have ─────

def build_page_meta(
    *,
    title: str,
    description: str,
    keywords: list[str],
    image_path: str,
    page_path: str,
    canonical_path: Optional[str] = None,
    image_alt: Optional[str] = None,
    og_type: str = "article",
    site: SiteConfig = DEFAULT_SITE,
) -> PageMeta:
    image_url = path_to_url(image_path)
    page_url = path_to_url(page_path)
    canonical_url = path_to_url(canonical_path or page_path)

    return PageMeta(
        title=title,
        description=description,
        keywords=keywords,
        canonical_url=canonical_url,
        image_url=image_url,
        page_url=page_url,
        image_alt=image_alt,
        og_type=og_type,
        site=site,
    )



from .main import *
from PyQt6.QtWidgets import QStackedWidget, QSplitter, QWidget, QVBoxLayout, QHBoxLayout, QLabel,QTreeWidget, QTreeWidgetItem, QHeaderView, QButtonGroup
from PyQt6.QtCore import QThread, pyqtSignal



from pathlib import Path
from .upload_document import upload_document
from .imports import *
from .upload_to_db import *
ROOT = Path("/srv/staging/pdfs")


def get_text_for_document(base_dir: Path):
    for txt in base_dir.rglob("text.txt"):
        return txt
    return None


def ingest_all():
    for pages_dir in ROOT.rglob("pages"):
        base_dir = pages_dir.parent

       
        print(f"📄 Processing: {base_dir}")

        base_path = str(base_dir)

        # ✅ Skip if already exists
        if document_exists_by_path(base_path):
            print("⏭️ Skipping (already exists)")
            continue

        # ✅ Step 1: insert document + pages ONLY
        doc_id = upload_document(base_dir)

        if not doc_id:
            print("⏭️ Skipped (already exists)")
            continue

        print(f"🧱 Inserted document {doc_id}")

        # ✅ Step 2: process pages independently
        process_pages(base_dir, doc_id)
        
        print(f"✅ Completed {doc_id}")

##        except Exception as e:
##            print(f"❌ Failed: {base_dir}")
##            print(e)
##            input(e)



from .enrich_record import *
# ---------------------------------------------------------------------------
# 6. PROCESSOR REGISTRY — handlers keyed by media kind
# ---------------------------------------------------------------------------

class ProcessorRegistry:
    """
    Register and dispatch processing functions by media kind.
    Each processor is a callable(record, config) -> None.
    Multiple processors per kind are supported (run in registration order).
    """

    def __init__(self):
        self._handlers = OrderedDict()   # kind -> [callable, ...]

    def register(self, kind, handler):
        """Register a handler for a media kind."""
        if kind not in self._handlers:
            self._handlers[kind] = []
        self._handlers[kind].append(handler)
        return handler

    def process(self, record, config):
        """Run all registered handlers for the record's kind."""
        kind = record.get("kind", "")
        for handler in self._handlers.get(kind, []):
            handler(record, config)

    def process_all(self, records, config):
        """Process an iterable of records."""
        for record in records:
            self.process(record, config)



from .discover import *
# ---------------------------------------------------------------------------
# 5. ENRICHMENT — resolve URLs, fill computed fields
# ---------------------------------------------------------------------------

def enrich_record(record, config):
    """
    Fill in all URL fields and canonical URL from paths.
    Mutates record in place and returns it.
    """
    mr = config.media_root
    sr = config.site_root

    record["dir_url"] = path_to_url(record["dir_path"], mr, sr)
    record["source_url"] = path_to_url(record["source_path"], mr, sr)
    record["html_url"] = path_to_url(record["html_path"], mr, sr) if record["html_path"] else ""
    record["thumbnail_url"] = path_to_url(record["thumbnail_path"], mr, sr)
    record["thumbnail_resized_url"] = path_to_url(record["thumbnail_resized_path"], mr, sr)
    record["poster_url"] = path_to_url(record["poster_path"], mr, sr)
    record["captions_url"] = path_to_url(record["captions_path"], mr, sr)

    # canonical: prefer html_url, fallback to dir_url
    if not record["canonical_url"]:
        meta_canonical = record["metadata"].get("canonical", "")
        if meta_canonical and not meta_canonical.endswith(
            (".png", ".jpg", ".jpeg", ".webp", ".gif", ".pdf")
        ):
            record["canonical_url"] = meta_canonical
        elif record["html_url"]:
            record["canonical_url"] = record["html_url"]
        else:
            record["canonical_url"] = record["dir_url"] + "/"

    # enrich page URLs for PDFs
    for page in record.get("pages", []):
        page["dir_url"] = path_to_url(page["dir_path"], mr, sr)
        page["image_url"] = path_to_url(page["image_path"], mr, sr)
        page["thumb_url"] = path_to_url(page["thumb_path"], mr, sr)
        page["html_url"] = path_to_url(page["html_path"], mr, sr)

    return record
